/*************************************************
*                                                *
*  EasyBMP Cross-Platform Windows Bitmap Library * 
*                                                *
*  Author: Paul Macklin                          *
*   email: pmacklin@math.uci.edu                 *
*                                                *
*    file: EasyBMP_StandardPalettes.h            *
*    date: 5-2-2005                              *
* version: 0.62                                  *
*                                                *
* description: Assists in writing color tables   *
*              for the BMP class                 *
*                                                *
*************************************************/

void CreateGreyscaleColorTable( RGBApixel** pColorPointer , int Depth )
{
 if( Depth != 1 && Depth != 4 && Depth != 8 )
 { return; }
 int i;
 int NumberOfColors = (int) pow(2,Depth);
 BYTE StepSize;
 if( Depth != 1 )
 {
  StepSize = 255/(NumberOfColors-1);
 }
 else
 {
  StepSize = 255;
 }
 
 for( i=0 ; i < NumberOfColors ; i++ )
 {
  BYTE TempColor = i*StepSize;
  (*pColorPointer)[i].Red = TempColor;
  (*pColorPointer)[i].Green = TempColor;
  (*pColorPointer)[i].Blue = TempColor;
  (*pColorPointer)[i].Alpha = 0;
 }
 
 return;
}

void Create1bitColorTable( RGBApixel** pColorPointer )
{
 (*pColorPointer)[0].Red = 0;
 (*pColorPointer)[0].Green = 0;
 (*pColorPointer)[0].Blue = 0;
 (*pColorPointer)[0].Alpha = 0;

 (*pColorPointer)[1].Red = 255;
 (*pColorPointer)[1].Green = 255;
 (*pColorPointer)[1].Blue = 255;
 (*pColorPointer)[1].Alpha = 0;
} 

void Create4bitColorTable( RGBApixel** pColorPointer )
{
 int i = 0;
 (*pColorPointer)[i].Red = 0;
 (*pColorPointer)[i].Green = 0;
 (*pColorPointer)[i].Blue = 0;
 i++; // 1
 (*pColorPointer)[i].Red = 128;
 (*pColorPointer)[i].Green = 0;
 (*pColorPointer)[i].Blue = 0;
 i++; // 2
 (*pColorPointer)[i].Red = 0;
 (*pColorPointer)[i].Green = 128;
 (*pColorPointer)[i].Blue = 0;
 i++; // 3
 (*pColorPointer)[i].Red = 128;
 (*pColorPointer)[i].Green = 128;
 (*pColorPointer)[i].Blue = 0;
 i++; // 4
 (*pColorPointer)[i].Red = 0;
 (*pColorPointer)[i].Green = 0;
 (*pColorPointer)[i].Blue = 128;
 i++; // 5
 (*pColorPointer)[i].Red = 128;
 (*pColorPointer)[i].Green = 0;
 (*pColorPointer)[i].Blue = 128;
 i++; // 6
 (*pColorPointer)[i].Red = 0;
 (*pColorPointer)[i].Green = 128;
 (*pColorPointer)[i].Blue = 128;
 i++; // 7
 (*pColorPointer)[i].Red = 128;
 (*pColorPointer)[i].Green = 128;
 (*pColorPointer)[i].Blue = 128;

 i++; // 8
 (*pColorPointer)[i].Red = 192;
 (*pColorPointer)[i].Green = 192;
 (*pColorPointer)[i].Blue = 192;
 i++; // 9
 (*pColorPointer)[i].Red = 255;
 (*pColorPointer)[i].Green = 0;
 (*pColorPointer)[i].Blue = 0;
 i++; // 10
 (*pColorPointer)[i].Red = 0;
 (*pColorPointer)[i].Green = 255;
 (*pColorPointer)[i].Blue = 0;
 i++; // 11
 (*pColorPointer)[i].Red = 255;
 (*pColorPointer)[i].Green = 255;
 (*pColorPointer)[i].Blue = 0;
 i++; // 12
 (*pColorPointer)[i].Red = 0;
 (*pColorPointer)[i].Green = 0;
 (*pColorPointer)[i].Blue = 255;
 i++; // 13
 (*pColorPointer)[i].Red = 255;
 (*pColorPointer)[i].Green = 0;
 (*pColorPointer)[i].Blue = 255;
 i++; // 14
 (*pColorPointer)[i].Red = 0;
 (*pColorPointer)[i].Green = 255;
 (*pColorPointer)[i].Blue = 255;
 i++; // 15
 (*pColorPointer)[i].Red = 255;
 (*pColorPointer)[i].Green = 255;
 (*pColorPointer)[i].Blue = 255;

 for( i=0 ; i < 16 ; i++ )
 {
  (*pColorPointer)[i].Alpha = 0;
 }
  
 return;
}

void Create8bitColorTable( RGBApixel** pColorPointer )
{
 int nLow, nHigh;
 BYTE unIndex;
 
 int i=0;
 (*pColorPointer)[i].Red = 0;
 (*pColorPointer)[i].Green = 0;
 (*pColorPointer)[i].Blue = 0;
 i++; // 1
 (*pColorPointer)[i].Red = 128;
 (*pColorPointer)[i].Green = 0;
 (*pColorPointer)[i].Blue = 0;
 i++; // 2
 (*pColorPointer)[i].Red = 0;
 (*pColorPointer)[i].Green = 128;
 (*pColorPointer)[i].Blue = 0;
 i++; // 3
 (*pColorPointer)[i].Red = 128;
 (*pColorPointer)[i].Green = 128;
 (*pColorPointer)[i].Blue = 0;
 i++; // 4
 (*pColorPointer)[i].Red = 0;
 (*pColorPointer)[i].Green = 0;
 (*pColorPointer)[i].Blue = 128;
 i++; // 5
 (*pColorPointer)[i].Red = 128;
 (*pColorPointer)[i].Green = 0;
 (*pColorPointer)[i].Blue = 128;
 i++; // 6
 (*pColorPointer)[i].Red = 0;
 (*pColorPointer)[i].Green = 128;
 (*pColorPointer)[i].Blue = 128;
 i++; // 7
 (*pColorPointer)[i].Red = 192;
 (*pColorPointer)[i].Green = 192;
 (*pColorPointer)[i].Blue = 192;
 i++; // 8
 (*pColorPointer)[i].Red = 192;
 (*pColorPointer)[i].Green = 220;
 (*pColorPointer)[i].Blue = 192;
 i++; // 9
 (*pColorPointer)[i].Red = 166;
 (*pColorPointer)[i].Green = 202;
 (*pColorPointer)[i].Blue = 240;
 i++; // 10
 (*pColorPointer)[i].Red = 64;
 (*pColorPointer)[i].Green = 32;
 (*pColorPointer)[i].Blue = 0;
 i++; // 11
 (*pColorPointer)[i].Red = 96;
 (*pColorPointer)[i].Green = 32;
 (*pColorPointer)[i].Blue = 0;
 i++; // 12
 (*pColorPointer)[i].Red = 128;
 (*pColorPointer)[i].Green = 32;
 (*pColorPointer)[i].Blue = 0;
 i++; // 13
 (*pColorPointer)[i].Red = 160;
 (*pColorPointer)[i].Green = 32;
 (*pColorPointer)[i].Blue = 0;
 i++; // 14
 (*pColorPointer)[i].Red = 192;
 (*pColorPointer)[i].Green = 32;
 (*pColorPointer)[i].Blue = 0;
 i++; // 15
 (*pColorPointer)[i].Red = 224;
 (*pColorPointer)[i].Green = 32;
 (*pColorPointer)[i].Blue = 0;

 nLow = 16;
 nHigh = nLow+7;
 // 16 to 23
 unIndex = 0;
 for(i=nLow ; i <= nHigh ; i++)
 {
  (*pColorPointer)[i].Red = unIndex;
  (*pColorPointer)[i].Green = 64;
  (*pColorPointer)[i].Blue = 0;
  unIndex += 32;
 }
 i = 24;

 nLow += 8;
 nHigh = nLow+7;
 // 24 to 31
 unIndex = 0;
 for(i=nLow ; i <= nHigh ; i++)
 {
  (*pColorPointer)[i].Red = unIndex;
  (*pColorPointer)[i].Green = 96;
  (*pColorPointer)[i].Blue = 0;
  unIndex += 32;
 }
 i = 32;
 
 nLow += 8;
 nHigh = nLow+7;
 // 32 to 39
 unIndex = 0;
 for(i=nLow ; i <= nHigh ; i++)
 {
  (*pColorPointer)[i].Red = unIndex;
  (*pColorPointer)[i].Green = 128;
  (*pColorPointer)[i].Blue = 0;
  unIndex += 32;
 }
 i = 40; 

 nLow += 8;
 nHigh = nLow+7;
 // 40 to 47
 unIndex = 0;
 for(i=nLow ; i <= nHigh ; i++)
 {
  (*pColorPointer)[i].Red = unIndex;
  (*pColorPointer)[i].Green = 160;
  (*pColorPointer)[i].Blue = 0;
  unIndex += 32;
 }
 i = nHigh+1; 

 nLow += 8;
 nHigh = nLow+7;
 // 48 to 55
 unIndex = 0;
 for(i=nLow ; i <= nHigh ; i++)
 {
  (*pColorPointer)[i].Red = unIndex;
  (*pColorPointer)[i].Green = 192;
  (*pColorPointer)[i].Blue = 0;
  unIndex += 32;
 }
 i = nHigh+1; 

 nLow += 8;
 nHigh = nLow+7;
 // 56 to 63
 unIndex = 0;
 for(i=nLow ; i <= nHigh ; i++)
 {
  (*pColorPointer)[i].Red = unIndex;
  (*pColorPointer)[i].Green = 224;
  (*pColorPointer)[i].Blue = 0;
  unIndex += 32;
 }
 i = nHigh+1; 

 nLow += 8;
 nHigh = nLow+7;
 // 64 to 71
 unIndex = 0;
 for(i=nLow ; i <= nHigh ; i++)
 {
  (*pColorPointer)[i].Red = unIndex;
  (*pColorPointer)[i].Green = 0;
  (*pColorPointer)[i].Blue = 64;
  unIndex += 32;
 }
 i = nHigh+1; 

 nLow += 8;
 nHigh = nLow+7;
 unIndex = 0;
 // 72 to 79
 for(i=nLow ; i <= nHigh ; i++)
 {
  (*pColorPointer)[i].Red = unIndex;
  (*pColorPointer)[i].Green = 32;
  (*pColorPointer)[i].Blue = 64;
  unIndex += 32;
 }

 nLow += 8;
 nHigh = nLow+7;
 // 80 to 87
 unIndex = 0;
 for(i=nLow ; i <=nHigh; i++)
 {
  (*pColorPointer)[i].Red = unIndex;
  (*pColorPointer)[i].Green = 64;
  (*pColorPointer)[i].Blue = 64;
  unIndex += 32;
 }

  // 88 to 95
  unIndex = 0;
  for(i=88 ; i <= 95 ; i++)
  {
   (*pColorPointer)[i].Red = unIndex;
   (*pColorPointer)[i].Green = 96;
   (*pColorPointer)[i].Blue = 64;
   unIndex += 32;
  }

  // 96 to 103
  unIndex = 0;
  for(i=96 ; i <= 103 ; i++)
  {
   (*pColorPointer)[i].Red = unIndex;
   (*pColorPointer)[i].Green = 128;
   (*pColorPointer)[i].Blue = 64;
   unIndex += 32;
  }
 
  //  104 to 111 
  unIndex = 0;
  for(i=104 ; i <= 111 ; i++)
  {
   (*pColorPointer)[i].Red = unIndex;
   (*pColorPointer)[i].Green = 160;
   (*pColorPointer)[i].Blue = 64;
   unIndex += 32;
  }

  // 112 to 119
  unIndex = 0;
  for(i=112 ; i <= 119 ; i++)
  {
   (*pColorPointer)[i].Red = unIndex;
   (*pColorPointer)[i].Green = 192;
   (*pColorPointer)[i].Blue = 64;
   unIndex += 32;
  }

  // 120 to 127
  unIndex = 0;
  for(i=120 ; i <= 127 ; i++)
  {
   (*pColorPointer)[i].Red = unIndex;
   (*pColorPointer)[i].Green = 224;
   (*pColorPointer)[i].Blue = 64;
   unIndex += 32;
  }

  nLow = 128;
  nHigh = nLow+7;
  // 128 to 135
  unIndex = 0;
  for(i=nLow ; i <= nHigh ; i++)
  {
   (*pColorPointer)[i].Red = unIndex;
   (*pColorPointer)[i].Green = 0;
   (*pColorPointer)[i].Blue = 128;
   unIndex += 32;
  }

  nLow += 8;
  nHigh += 8;
  // 136 to 143
  unIndex = 0;
  for(i=nLow ; i <= nHigh ; i++)
  {
   (*pColorPointer)[i].Red = unIndex;
   (*pColorPointer)[i].Green = 32;
   (*pColorPointer)[i].Blue = 128;
   unIndex += 32;
  }

  nLow += 8;
  nHigh += 8;
  // 144 to 151
  unIndex = 0;
  for(i=nLow ; i <= nHigh ; i++)
  {
   (*pColorPointer)[i].Red = unIndex;
   (*pColorPointer)[i].Green = 64;
   (*pColorPointer)[i].Blue = 128;
   unIndex += 32;
  }

  nLow += 8;
  nHigh += 8;
  // 152 to 159
  unIndex = 0;
  for(i=nLow ; i <= nHigh ; i++)
  {
   (*pColorPointer)[i].Red = unIndex;
   (*pColorPointer)[i].Green = 96;
   (*pColorPointer)[i].Blue = 128;
   unIndex += 32;
  }

  nLow += 8;
  nHigh += 8;
  // 160 to 167
  unIndex = 0;
  for(i=nLow ; i <= nHigh ; i++)
  {
   unsigned int unIndex = 0;
   (*pColorPointer)[i].Red = unIndex;
   (*pColorPointer)[i].Green = 128;
   (*pColorPointer)[i].Blue = 128;
   unIndex += 32;
  }

  nLow += 8;
  nHigh += 8;
  // 168 to 175
  unIndex = 0;
  for(i=nLow ; i <= nHigh ; i++)
  {
   (*pColorPointer)[i].Red = unIndex;
   (*pColorPointer)[i].Green = 160;
   (*pColorPointer)[i].Blue = 128;
   unIndex += 32;
  }

  nLow += 8;
  nHigh += 8;
  // 176 to 183
  unIndex = 0;
  for(i=nLow ; i <= nHigh ; i++)
  {
   (*pColorPointer)[i].Red = unIndex;
   (*pColorPointer)[i].Green = 192;
   (*pColorPointer)[i].Blue = 128;
   unIndex += 32;
  }

  nLow += 8;
  nHigh += 8;
  // 184 to 191
  unIndex = 0;
  for(i=nLow ; i <= nHigh ; i++)
  {
   (*pColorPointer)[i].Red = unIndex;
   (*pColorPointer)[i].Green = 224;
   (*pColorPointer)[i].Blue = 128;
   unIndex += 32;
  }

  nLow += 8;
  nHigh += 8;
  // 192 to 199
  unIndex = 0;
  for(i=nLow ; i <= nHigh ; i++)
  {
   (*pColorPointer)[i].Red = unIndex;
   (*pColorPointer)[i].Green = 0;
   (*pColorPointer)[i].Blue = 192;
   unIndex += 32;
  }

  nLow += 8;
  nHigh += 8;
  // 200 to 207
  unIndex = 0;
  for(i=nLow ; i <= nHigh ; i++)
  {
   (*pColorPointer)[i].Red = unIndex;
   (*pColorPointer)[i].Green = 32;
   (*pColorPointer)[i].Blue = 192;
   unIndex += 32;
  }

  nLow += 8;
  nHigh += 8;
  // 208 to 215
  unIndex = 0;
  for(i=nLow ; i <= nHigh ; i++)
  {
   (*pColorPointer)[i].Red = unIndex;
   (*pColorPointer)[i].Green = 64;
   (*pColorPointer)[i].Blue = 192;
   unIndex += 32;
  }

  nLow += 8;
  nHigh += 8;
  // 216 to 223 
  unIndex = 0;
  for(i=nLow ; i <= nHigh ; i++)
  {
   (*pColorPointer)[i].Red = unIndex;
   (*pColorPointer)[i].Green = 96;
   (*pColorPointer)[i].Blue = 192;
   unIndex += 32;
  }

  nLow += 8;
  nHigh += 8;
  // 224 to 231
  unIndex = 0;
  for(i=nLow ; i <= nHigh ; i++)
  {
   (*pColorPointer)[i].Red = unIndex;
   (*pColorPointer)[i].Green = 128;
   (*pColorPointer)[i].Blue = 192;
   unIndex += 32;
  }

  nLow += 8;
  nHigh += 8;
  unIndex = 0;
  // 232 to 239
  for(i=nLow ; i <= nHigh ; i++)
  {
   (*pColorPointer)[i].Red = unIndex;
   (*pColorPointer)[i].Green = 160;
   (*pColorPointer)[i].Blue = 192;
   unIndex += 32;
  }

  nLow += 8;
  nHigh = 245;
  unIndex = 0;
  // 240 to 245 
 for(i=nLow ; i <= nHigh ; i++)
 {
  (*pColorPointer)[i].Red = unIndex;
  (*pColorPointer)[i].Green = 192;
  (*pColorPointer)[i].Blue = 192;
  unIndex += 32;
 }

 i=246;
 (*pColorPointer)[i].Red = 255;
 (*pColorPointer)[i].Green = 251;
 (*pColorPointer)[i].Blue = 240;
 i++; // 247
 (*pColorPointer)[i].Red = 160;
 (*pColorPointer)[i].Green = 160;
 (*pColorPointer)[i].Blue = 164;
 i++; // 248
 i = 248; // temp
 (*pColorPointer)[i].Red = 128;
 (*pColorPointer)[i].Green = 128;
 (*pColorPointer)[i].Blue = 128;
 i++; // 249
 (*pColorPointer)[i].Red = 255;
 (*pColorPointer)[i].Green = 0;
 (*pColorPointer)[i].Blue = 0;
 i++; // 250
 (*pColorPointer)[i].Red = 0;
 (*pColorPointer)[i].Green = 255;
 (*pColorPointer)[i].Blue = 0;
 i++; // 251
 (*pColorPointer)[i].Red = 255;
 (*pColorPointer)[i].Green = 255;
 (*pColorPointer)[i].Blue = 0;
 i++; // 252
 (*pColorPointer)[i].Red = 0;
 (*pColorPointer)[i].Green = 0;
 (*pColorPointer)[i].Blue = 255;
 i++; // 253
 (*pColorPointer)[i].Red = 255;
 (*pColorPointer)[i].Green = 0;
 (*pColorPointer)[i].Blue = 255;
 i++; // 254
 (*pColorPointer)[i].Red = 0;
 (*pColorPointer)[i].Green = 255;
 (*pColorPointer)[i].Blue = 255;
 i++; // 255
 (*pColorPointer)[i].Red = 255;
 (*pColorPointer)[i].Green = 255;
 (*pColorPointer)[i].Blue = 255;

 for( i=0 ; i < 16 ; i++ )
 {
  (*pColorPointer)[i].Alpha = 0;
 }
 
 return;
}

void CreateStandardColorTable(  RGBApixel** pColorPointer , int Depth )
{
 if( Depth == 1 )
 {
  Create1bitColorTable( pColorPointer );
  return;
 }
 if( Depth == 4 )
 {
  Create4bitColorTable( pColorPointer );
  return;
 }
 if( Depth == 8 )
 {
  Create8bitColorTable( pColorPointer );
  return;
 }
 return;
}
